# -*- coding: ascii -*-

###########################################################################
# clive, video extraction utility
# Copyright (C) 2007-2008 Toni Gundogdu
#
# clive is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 0.1.2-1307 USA
###########################################################################

## The classes for parsing rss/atom feeds

import urllib2

try:
    import feedparser as fparser
    import snack as newt
except ImportError, err:
    raise SystemExit('error: %s' % err)

from clive.unicode import tostr

__all__ = ['FeedParser']

## The feed parsing class
class FeedParser:

    ## Constructor
    def __init__(self, opts, say, proxy):
        self._opts = opts
        self._say = say
        self._proxy = proxy

    ## Reap feeds and display selection lists
    def parse(self, raw_urls):
        fparser.USER_AGENT = self._opts.http_agent
        h = urllib2.ProxyHandler(self._proxy)
        # Reap URLs
        feeds = []
        for url in raw_urls:
            self._say('feed: %s' % url)
            d = fparser.parse(url, handlers=[h])
            try:
                d.feed.title;
                feeds.append(d)
            except AttributeError:
                self._say('error: invalid rss/atom feed')
        # Display feeds and return user selection (video URLs)
        if len(feeds) == 0:
            raise SystemExit('error: no-data; check rss/atom urls')
        u = DisplayFeeds(feeds, self._opts)
        return u.get_selection()


## The class that displays the feeds with found video URLs
class DisplayFeeds:

    ## Constructor
    def __init__(self, feeds, opts):
        self._last = None # Selection before hitting "Refresh"
        self._feeds = feeds
        self._opts = opts
        while 1:
            self._selection = self._main()
            if self._selection: break

    ## Returns the user selection of videos to be extracted
    def get_selection(self):
        return self._selection

    def _main(self):
        sel = []
        try:
            scr = newt.SnackScreen()
            sel = self._show_feeds(scr)
        finally:
            scr.finish()
        return sel

    def _ok(self):
        return self._ct.getSelection()

    def _cancel(self):
        raise SystemExit('Cancelled.')

    def _refresh(self):
        self._last = self._ct.getSelection()
        return None

    def _show_feeds(self, scr):
        w = scr.width-24
        ww = w-8
        ct = newt.CheckboxTree(scr.height/2, scroll=1)
        for (i,d) in enumerate(self._feeds):
            t = d.feed.title[:w]
            if len(t) == w: t += '..'
            t += (' (%d)' % len(d.entries))
            t = tostr(t, self._opts)
            ct.append(t)
            for e in d.entries:
                t = e.title[:ww]
                if len(t) == ww: t += '..'
                sel = False
                if self._last and e.link in self._last:
                    sel = True
                t = tostr(t, self._opts)
                ct.addItem(t, (i, newt.snackArgs['append']),
                    e.link, selected=sel)
        g = newt.GridForm(scr, 'RSS/Atom Feeds: Videos found', 1,2)
        g.add(ct, col=0, row=0, padding=(0,0,0,1))
        b = newt.ButtonBar(scr,
            [('Refresh',2),('Cancel',0), ('OK - Extract',1)], compact=0)
        g.add(b, col=0, row=1, padding=(0,0,0,0))
        self._ct = ct
        d = {0:self._cancel, 1:self._ok, 2:self._refresh, None:self._ok}
        return d.get(b.buttonPressed(g.runOnce()))()
